lexer grammar syslogLexer;

tokens
{
// Common (language independent) tokens with dummies to keep the same numbering
  DUMMY1,
  DUMMY2,
  DUMMY3,
  HIDDEN_CONTENT_HOOK,
  AUX_COMMENT,
  VOCAB,
  WHITE_SPACE,
  ENDLINE,
  TS_ACTION,
// Verilog operators
  AT,               // @
  COLON,            // :
  COMMA,            // ,
  DOT,              // .
  ASSIGN,           // =
  MINUS,            // -
  LBRACK,           // [
  RBRACK,           // ]
  LCURLY,           // {
  RCURLY,           // }
  LPAREN,           // (
  RPAREN,           // )
  DELAY,            // #
  QUESTION,         // ?
  SEMI,             // ;
  PLUS,             // +
  LNOT,             // !
  BNOT,             // ~
  BAND,             // &
  RNAND,            // ~&
  BOR,              // |
  RNOR,             // ~|
  BXOR,             // ^
  RXNOR,            // ~^ or ^~
  POWER,            // **
  MULT,             // *
  DIV,              // /
  MOD,              // %
  EQUAL,            // ==
  NOT_EQ,           // !=
  NOT_EQ_CASE,      // !==
  EQ_CASE,          // ===
  SAND,             // &&&
  LAND,             // &&
  LOR,              // ||
  LT_,              // <
  LE,               // <=
  GT,               // >
  GE,               // >=
  SR,               // >>
  SL,               // <<
  ASR,              // >>>
  ASL,              // <<<
  TRIGGER,          // ->
  PPATH,            // =>
  FPATH,            // *>
  LATTR,            // (*
  RATTR,            // *)
  PS_UP,            // +:
  PS_DOWN,          // -:
// SystemVerilog operators
  DOLLAR,           // $
  PLUS_ASSIGN,      // +=
  MINUS_ASSIGN,     // -=
  MULT_ASSIGN,      // *=
  DIV_ASSIGN,       // /=
  MOD_ASSIGN,       // %=
  BAND_ASSIGN,      // &=
  BOR_ASSIGN,       // |=
  BXOR_ASSIGN,      // ^=
  SR_ASSIGN,        // >>=
  SL_ASSIGN,        // <<=
  ASR_ASSIGN,       // >>>=
  ASL_ASSIGN,       // <<<=
  INC,              // ++
  DEC,              // --
  WILDCARD_EQ,      // ==?
  WILDCARD_NOT_EQ,  // !=?
  NBLK_TRIGGER,     // ->>
  ATAT,             // @@
  CYCLE_DELAY,      // ##
  OL_IMPL,          // |->
  NOL_IMPL,         // |=>
  STRUCT_LCURLY,    // '{
  CAST,             // '
  WEIGHT,           // :=
  SPLIT_WEIGHT,     // :/
  SCOPE,            // ::
  CONS_REP_LBRACK,  // [*
  NCONS_REP_LBRACK, // [=
  GOTO_REP_LBRACK,  // [->
  LEQ,              // <->
  NOL_FOLLOWED_BY,  // #-#
  OL_FOLLOWED_BY,   // #=#
// Identifier tokens
  C_IDENTIFIER,       // [a-z,A-Z,_] ( [a-z,A-Z,0-9,_] )* (keywords filtered out)
  IDENTIFIER_EX,      // [a-z,A-Z,_] ( [a-z,A-Z,0-9,_,$] )+ (with at least one $)
  ESCAPED_IDENTIFIER, // \ (any nonwhite ascii)+
  SYSTF_NAME,         // $ ( [a-z,A-Z,0-9,_,$] )+ (builtin keywords/specials filtered out)
// Numerical tokens
  STRING,
  BASED_DECIMAL,
  BASED_BINARY,
  BASED_OCTAL,
  BASED_HEX,
  UNSIGNED_NUMBER,
  REAL_NUMBER,
  UNBASED_NUMBER,      // SV only: '0, '1, 'x, 'X, 'z or 'Z
  TIME_LITERAL,        // SV only: unsigned(.unsigned)?(s|ms|us|ns|ps|fs|step)
// all SystemVerilog keywords in alphabetical order
  KWD_accept_on,           //1800-2009
  KWD_alias,               //1800-2005
  KWD_always,              //1364-1995
  KWD_always_comb,         //1800-2005
  KWD_always_ff,           //1800-2005
  KWD_always_latch,        //1800-2005
  KWD_and,                 //1364-1995
  KWD_assert,              //1800-2005
  KWD_assign,              //1364-1995
  KWD_assume,              //1800-2005
  KWD_automatic,           //1364-2001
  KWD_before,              //1800-2005
  KWD_begin,               //1364-1995
  KWD_bind,                //1800-2005
  KWD_bins,                //1800-2005
  KWD_binsof,              //1800-2005
  KWD_bit,                 //1800-2005
  KWD_break,               //1800-2005
  KWD_buf,                 //1364-1995
  KWD_bufif0,              //1364-1995
  KWD_bufif1,              //1364-1995
  KWD_byte,                //1800-2005
  KWD_case,                //1364-1995
  KWD_casex,               //1364-1995
  KWD_casez,               //1364-1995
  KWD_cell,                //1364-2001 (libmap)
  KWD_chandle,             //1800-2005
  KWD_checker,             //1800-2009
  KWD_class,               //1800-2005
  KWD_clocking,            //1800-2005
  KWD_cmos,                //1364-1995
  KWD_config,              //1364-2001 (libmap)
  KWD_const,               //1800-2005
  KWD_constraint,          //1800-2005
  KWD_context,             //1800-2005
  KWD_continue,            //1800-2005
  KWD_cover,               //1800-2005
  KWD_covergroup,          //1800-2005
  KWD_coverpoint,          //1800-2005
  KWD_cross,               //1800-2005
  KWD_deassign,            //1364-1995
  KWD_default,             //1364-1995
  KWD_defparam,            //1364-1995
  KWD_design,              //1364-2001 (libmap)
  KWD_disable,             //1364-1995
  KWD_dist,                //1800-2005
  KWD_do,                  //1800-2005
  KWD_edge,                //1364-1995
  KWD_else,                //1364-1995
  KWD_end,                 //1364-1995
  KWD_endcase,             //1364-1995
  KWD_endchecker,          //1800-2009 
  KWD_endclass,            //1800-2005
  KWD_endclocking,         //1800-2005
  KWD_endconfig,           //1364-2001 (libmap)
  KWD_endfunction,         //1364-1995
  KWD_endgenerate,         //1364-2001
  KWD_endgroup,            //1800-2005
  KWD_endinterface,        //1800-2005
  KWD_endmodule,           //1364-1995
  KWD_endpackage,          //1800-2005
  KWD_endprimitive,        //1364-1995
  KWD_endprogram,          //1800-2005
  KWD_endproperty,         //1800-2005
  KWD_endsequence,         //1800-2005
  KWD_endspecify,          //1364-1995
  KWD_endtable,            //1364-1995
  KWD_endtask,             //1364-1995
  KWD_enum,                //1800-2005
  KWD_event,               //1364-1995
  KWD_eventually,          //1800-2009
  KWD_expect,              //1800-2005
  KWD_export,              //1800-2005
  KWD_extends,             //1800-2005
  KWD_extern,              //1800-2005
  KWD_final,               //1800-2005
  KWD_first_match,         //1800-2005
  KWD_for,                 //1364-1995
  KWD_force,               //1364-1995
  KWD_foreach,             //1800-2005
  KWD_forever,             //1364-1995
  KWD_fork,                //1364-1995
  KWD_forkjoin,            //1800-2005
  KWD_function,            //1364-1995
  KWD_generate,            //1364-2001
  KWD_genvar,              //1364-2001
  KWD_global,              //1800-2009
  KWD_highz0,              //1364-1995
  KWD_highz1,              //1364-1995
  KWD_if,                  //1364-1995
  KWD_iff,                 //1800-2005
  KWD_ifnone,              //1364-1995
  KWD_ignore_bins,         //1800-2005
  KWD_illegal_bins,        //1800-2005
  KWD_implements,          //1800-2012
  KWD_implies,             //1800-2009
  KWD_import,              //1800-2005
  KWD_incdir,              //1364-2001 (libmap)
  KWD_include,             //1364-2001 (libmap)
  KWD_initial,             //1364-1995
  KWD_inout,               //1364-1995
  KWD_input,               //1364-1995
  KWD_inside,              //1800-2005
  KWD_instance,            //1364-2001 (libmap)
  KWD_int,                 //1800-2005
  KWD_integer,             //1364-1995
  KWD_interconnect,        //1800-2012
  KWD_interface,           //1800-2005
  KWD_intersect,           //1800-2005
  KWD_join,                //1364-1995
  KWD_join_any,            //1800-2005
  KWD_join_none,           //1800-2005
  KWD_large,               //1364-1995
  KWD_let,                 //1800-2009
  KWD_liblist,             //1364-2001 (libmap)
  KWD_library,             //1364-2001 (libmap)
  KWD_local,               //1800-2005
  KWD_localparam,          //1364-2001
  KWD_logic,               //1800-2005
  KWD_longint,             //1800-2005
  KWD_macromodule,         //1364-1995
  KWD_matches,             //1800-2005
  KWD_medium,              //1364-1995
  KWD_modport,             //1800-2005
  KWD_module,              //1364-1995
  KWD_nand,                //1364-1995
  KWD_negedge,             //1364-1995
  KWD_nettype,             //1800-2012
  KWD_new,                 //1800-2005
  KWD_nexttime,            //1800-2009
  KWD_nmos,                //1364-1995
  KWD_nor,                 //1364-1995
  KWD_noshowcancelled,     //1364-2001
  KWD_not,                 //1364-1995
  KWD_notif0,              //1364-1995
  KWD_notif1,              //1364-1995
  KWD_null,                //1800-2005
  KWD_or,                  //1364-1995
  KWD_output,              //1364-1995
  KWD_package,             //1800-2005
  KWD_packed,              //1800-2005
  KWD_parameter,           //1364-1995
  KWD_pmos,                //1364-1995
  KWD_posedge,             //1364-1995
  KWD_primitive,           //1364-1995
  KWD_priority,            //1800-2005
  KWD_program,             //1800-2005
  KWD_property,            //1800-2005
  KWD_protected,           //1800-2005
  KWD_pull0,               //1364-1995
  KWD_pull1,               //1364-1995
  KWD_pulldown,            //1364-1995
  KWD_pullup,              //1364-1995
  KWD_pulsestyle_ondetect, //1364-2001
  KWD_pulsestyle_onevent,  //1364-2001
  KWD_pure,                //1800-2005
  KWD_rand,                //1800-2005
  KWD_randc,               //1800-2005
  KWD_randcase,            //1800-2005
  KWD_randsequence,        //1800-2005
  KWD_rcmos,               //1364-1995
  KWD_real,                //1364-1995
  KWD_realtime,            //1364-1995
  KWD_ref,                 //1800-2005
  KWD_reg,                 //1364-1995
  KWD_reject_on,           //1800-2009
  KWD_release,             //1364-1995
  KWD_repeat,              //1364-1995
  KWD_restrict,            //1800-2009
  KWD_return,              //1800-2005
  KWD_rnmos,               //1364-1995
  KWD_rpmos,               //1364-1995
  KWD_rtran,               //1364-1995
  KWD_rtranif0,            //1364-1995
  KWD_rtranif1,            //1364-1995
  KWD_s_always,            //1800-2009
  KWD_s_eventually,        //1800-2009
  KWD_s_nexttime,          //1800-2009
  KWD_s_until,             //1800-2009
  KWD_s_until_with,        //1800-2009
  KWD_scalared,            //1364-1995
  KWD_sequence,            //1800-2005
  KWD_shortint,            //1800-2005
  KWD_shortreal,           //1800-2005
  KWD_showcancelled,       //1364-2001
  KWD_signed,              //1364-2001
  KWD_small,               //1364-1995
  KWD_soft,                //1800-2012
  KWD_solve,               //1800-2005
  KWD_specify,             //1364-1995
  KWD_specparam,           //1364-1995
  KWD_static,              //1800-2005
  KWD_string,              //1800-2005
  KWD_strong,              //1800-2009
  KWD_strong0,             //1364-1995
  KWD_strong1,             //1364-1995
  KWD_struct,              //1800-2005
  KWD_super,               //1800-2005
  KWD_supply0,             //1364-1995
  KWD_supply1,             //1364-1995
  KWD_sync_accept_on,      //1800-2009
  KWD_sync_reject_on,      //1800-2009
  KWD_table,               //1364-1995
  KWD_tagged,              //1800-2005
  KWD_task,                //1364-1995
  KWD_this,                //1800-2005
  KWD_throughout,          //1800-2005
  KWD_time,                //1364-1995
  KWD_timeprecision,       //1800-2005
  KWD_timeunit,            //1800-2005
  KWD_tran,                //1364-1995
  KWD_tranif0,             //1364-1995
  KWD_tranif1,             //1364-1995
  KWD_tri,                 //1364-1995
  KWD_tri0,                //1364-1995
  KWD_tri1,                //1364-1995
  KWD_triand,              //1364-1995
  KWD_trior,               //1364-1995
  KWD_trireg,              //1364-1995
  KWD_type,                //1800-2005
  KWD_typedef,             //1800-2005
  KWD_union,               //1800-2005
  KWD_unique,              //1800-2005
  KWD_unique0,             //1800-2009
  KWD_unsigned,            //1364-2001
  KWD_until,               //1800-2009
  KWD_until_with,          //1800-2009
  KWD_untyped,             //1800-2009
  KWD_use,                 //1364-2001 (libmap)
  KWD_uwire,               //1364-2005
  KWD_var,                 //1800-2005
  KWD_vectored,            //1364-1995
  KWD_virtual,             //1800-2005
  KWD_void,                //1800-2005
  KWD_wait,                //1364-1995
  KWD_wait_order,          //1800-2005
  KWD_wand,                //1364-1995
  KWD_weak,                //1800-2009
  KWD_weak0,               //1364-1995
  KWD_weak1,               //1364-1995
  KWD_while,               //1364-1995
  KWD_wildcard,            //1800-2005
  KWD_wire,                //1364-1995
  KWD_with,                //1800-2005
  KWD_within,              //1800-2005
  KWD_wor,                 //1364-1995
  KWD_xnor,                //1364-1995
  KWD_xor,                 //1364-1995
//system literals - timing checks (all are specials recognized by lexer)
  SYSTF_setup,
  SYSTF_hold,
  SYSTF_setuphold,
  SYSTF_recovery,
  SYSTF_removal,
  SYSTF_recrem,
  SYSTF_skew,
  SYSTF_timeskew,
  SYSTF_fullskew,
  SYSTF_period,
  SYSTF_width,
  SYSTF_nochange,
//system literals - functions in alphabetical order (only keywords/specials are recognized by lexer)
  SYSTF_acos,
  SYSTF_acosh,
  SYSTF_asin,
  SYSTF_asinh,
  SYSTF_atan,
  SYSTF_atan2,
  SYSTF_atanh,
  SYSTF_bits,
  SYSTF_bitstoreal,
  SYSTF_bitstoshortreal,
  SYSTF_cast, //also a task
  SYSTF_ceil,
  SYSTF_changed,
  SYSTF_changed_gclk,
  SYSTF_changing_gclk,
  SYSTF_clog2,
  SYSTF_cos,
  SYSTF_cosh,
  SYSTF_countbits,
  SYSTF_countdrivers,
  SYSTF_countones,
  SYSTF_coverage_control,
  SYSTF_coverage_get,
  SYSTF_coverage_get_max,
  SYSTF_coverage_merge,
  SYSTF_coverage_save,
  SYSTF_dimensions,
  SYSTF_dist_chi_square,
  SYSTF_dist_erlang,
  SYSTF_dist_exponential,
  SYSTF_dist_normal,
  SYSTF_dist_poisson,
  SYSTF_dist_t,
  SYSTF_dist_uniform,
  SYSTF_exp,
  SYSTF_falling_gclk,
  SYSTF_fell,
  SYSTF_fell_gclk,
  SYSTF_feof,
  SYSTF_ferror,
  SYSTF_fgetc,
  SYSTF_fgets,
  SYSTF_floor,
  SYSTF_fopen,
  SYSTF_fread,
  SYSTF_fscanf,
  SYSTF_fseek,
  SYSTF_ftell,
  SYSTF_future_gclk,
  SYSTF_get_coverage,
  SYSTF_global_clock,
  SYSTF_high,
  SYSTF_hypot,
  SYSTF_increment,
  SYSTF_isunbounded,
  SYSTF_isunknown,
  SYSTF_itor,
  SYSTF_left,
  SYSTF_ln,
  SYSTF_load_coverage_db,
  SYSTF_log10,
  SYSTF_low,
  SYSTF_onehot,
  SYSTF_onehot0,
  SYSTF_past,
  SYSTF_past_gclk,
  SYSTF_pow,
  SYSTF_psprintf,
  SYSTF_q_full,
  SYSTF_random,
  SYSTF_realtime,
  SYSTF_realtobits,
  SYSTF_rewind,
  SYSTF_right,
  SYSTF_rising_gclk,
  SYSTF_rose,
  SYSTF_rose_gclk,
  SYSTF_rtoi,
  SYSTF_sampled,
  SYSTF_set_coverage_db_name,
  SYSTF_sformatf,
  SYSTF_shortrealtobits,
  SYSTF_signed,
  SYSTF_sin,
  SYSTF_sinh,
  SYSTF_size,
  SYSTF_sqrt,
  SYSTF_sscanf,
  SYSTF_stable,
  SYSTF_stable_gclk,
  SYSTF_steady_gclk,
  SYSTF_stime,
  SYSTF_system,
  SYSTF_tan,
  SYSTF_tanh,
  SYSTF_test4plusargs,
  SYSTF_time,
  SYSTF_typename,
  SYSTF_ungetc,
  SYSTF_unpacked_dimensions,
  SYSTF_unsigned,
  SYSTF_urandom,
  SYSTF_urandom_range,
  SYSTF_value4plusargs,
//system literals - tasks in alphabetical order (only keywords/specials are recognized by lexer)
  SYSTF_assertcontrol,
  SYSTF_assertfailoff,
  SYSTF_assertfailon,
  SYSTF_assertkill,
  SYSTF_assertnonvacuouson,
  SYSTF_assertoff,
  SYSTF_asserton,
  SYSTF_assertpassoff,
  SYSTF_assertpasson,
  SYSTF_assertvacuousoff,
  SYSTF_async4and4array,
  SYSTF_async4and4plane,
  SYSTF_async4nand4array,
  SYSTF_async4nand4plane,
  SYSTF_async4nor4array,
  SYSTF_async4nor4plane,
  SYSTF_async4or4array,
  SYSTF_async4or4plane,
  //SYSTF_cast, //already defined among functions, can be used as task too
  SYSTF_display,
  SYSTF_displayb,
  SYSTF_displayh,
  SYSTF_displayo,
  SYSTF_dumpall,
  SYSTF_dumpfile,
  SYSTF_dumpflush,
  SYSTF_dumplimit,
  SYSTF_dumpoff,
  SYSTF_dumpon,
  SYSTF_dumpports,
  SYSTF_dumpportsall,
  SYSTF_dumpportsflush,
  SYSTF_dumpportslimit,
  SYSTF_dumpportsoff,
  SYSTF_dumpportson,
  SYSTF_dumpvars,
  SYSTF_error, //special - elaboration task (recognized by lexer)
  SYSTF_exit,
  SYSTF_fatal, //special - elaboration task (recognized by lexer)
  SYSTF_fclose,
  SYSTF_fdisplay,
  SYSTF_fdisplayb,
  SYSTF_fdisplayh,
  SYSTF_fdisplayo,
  SYSTF_fflush,
  SYSTF_finish,
  SYSTF_fmonitor,
  SYSTF_fmonitorb,
  SYSTF_fmonitorh,
  SYSTF_fmonitoro,
  SYSTF_fstrobe,
  SYSTF_fstrobeb,
  SYSTF_fstrobeh,
  SYSTF_fstrobeo,
  SYSTF_fwrite,
  SYSTF_fwriteb,
  SYSTF_fwriteh,
  SYSTF_fwriteo,
  SYSTF_info, //special - elaboration task (recognized by lexer)
  SYSTF_monitor,
  SYSTF_monitorb,
  SYSTF_monitorh,
  SYSTF_monitoro,
  SYSTF_monitoroff,
  SYSTF_monitoron,
  SYSTF_printtimescale,
  SYSTF_q_add,
  SYSTF_q_exam,
  SYSTF_q_initialize,
  SYSTF_q_remove,
  SYSTF_readmemb,
  SYSTF_readmemh,
  SYSTF_sdf_annotate,
  SYSTF_sformat,
  SYSTF_signal_force, //MTI
  SYSTF_signal_release, //MTI
  SYSTF_stacktrace, //MTI/NCSim
  SYSTF_stop,
  SYSTF_strobe,
  SYSTF_strobeb,
  SYSTF_strobeh,
  SYSTF_strobeo,
  SYSTF_swrite,
  SYSTF_swriteb,
  SYSTF_swriteh,
  SYSTF_swriteo,
  SYSTF_sync4and4array,
  SYSTF_sync4and4plane,
  SYSTF_sync4nand4array,
  SYSTF_sync4nand4plane,
  SYSTF_sync4nor4array,
  SYSTF_sync4nor4plane,
  SYSTF_sync4or4array,
  SYSTF_sync4or4plane,
  SYSTF_timeformat,
  SYSTF_warning, //special - elaboration task (recognized by lexer)
  SYSTF_write,
  SYSTF_writeb,
  SYSTF_writeh,
  SYSTF_writememb,
  SYSTF_writememh,
  SYSTF_writeo,
// SystemVerilog system keywords (turned to SYSTF_NAME in Verilog mode)
  SYSTF_root,
  SYSTF_unit,
// Tokens for preprocessor/filter only
  GRAVE,                // `
  MACRO_CONTINUE,       // \ (followed by newline or eof, but without it)
  MACRO_CONCAT,         // ``
  MACRO_QUOTE,          // `"
  MACRO_ESCAPED_QUOTE,  // `\`"
  MACRO,                // ` ( c_id | id_ex | esc_id ) (directives filtered out)
  STRING_PART,          // broken token - open string stopped on escaped newline
  BASED_DECIMAL_PART,   // broken token - just decimal base with no number
  BASED_BINARY_PART,    // broken token - just binary base with no number
  BASED_OCTAL_PART,     // broken token - just octal base with no number
  BASED_HEX_PART,       // broken token - just hex base with no number
  USELIB_ESCAPED_WHITESPACE, // \ (single whitespace char)
  //WHITE_SPACE,             //added to Common
  //ENDLINE,                 //added to Common
  SL_COMMENT,              //AUX_COMMENT and VOCAB are among common token types
  ML_COMMENT,
// builtin compiler directives in alphabetical order
  DIR___FILE__,
  DIR___LINE__,
  DIR_begin_keywords,
  DIR_celldefine,
  DIR_default_nettype,
  DIR_define,
  DIR_else,
  DIR_elsif,
  DIR_end_keywords,
  DIR_endcelldefine,
  DIR_endif,
  DIR_endprotect,
  DIR_ifdef,
  DIR_ifndef,
  DIR_include,
  DIR_line,
  DIR_nounconnected_drive,
  DIR_nouselib,
  DIR_pragma,
  DIR_protect,
  DIR_resetall,
  DIR_syncad_smartmodel_begin,
  DIR_syncad_smartmodel_end,
  DIR_timescale,
  DIR_unconnected_drive,
  DIR_undef,
  DIR_uselib
}

//keywords:
KWD_accept_on: 'accept_on';
KWD_alias: 'alias';
KWD_always: 'always';
KWD_always_comb: 'always_comb';
KWD_always_ff: 'always_ff';
KWD_always_latch: 'always_latch';
KWD_and: 'and';
KWD_assert: 'assert';
KWD_assign: 'assign';
KWD_assume: 'assume';
KWD_automatic: 'automatic';
KWD_before: 'before';
KWD_begin: 'begin';
KWD_bind: 'bind';
KWD_bins: 'bins';
KWD_binsof: 'binsof';
KWD_bit: 'bit';
KWD_break: 'break';
KWD_buf: 'buf';
KWD_bufif0: 'bufif0';
KWD_bufif1: 'bufif1';
KWD_byte: 'byte';
KWD_case: 'case';
KWD_casex: 'casex';
KWD_casez: 'casez';
KWD_cell: 'cell';
KWD_chandle: 'chandle';
KWD_checker: 'checker';
KWD_class: 'class';
KWD_clocking: 'clocking';
KWD_cmos: 'cmos';
KWD_config: 'config';
KWD_const: 'const';
KWD_constraint: 'constraint';
KWD_context: 'context';
KWD_continue: 'continue';
KWD_cover: 'cover';
KWD_covergroup: 'covergroup';
KWD_coverpoint: 'coverpoint';
KWD_cross: 'cross';
KWD_deassign: 'deassign';
KWD_default: 'default';
KWD_defparam: 'defparam';
KWD_design: 'design';
KWD_disable: 'disable';
KWD_dist: 'dist';
KWD_do: 'do';
KWD_edge: 'edge';
KWD_else: 'else';
KWD_end: 'end';
KWD_endcase: 'endcase';
KWD_endchecker: 'endchecker';
KWD_endclass: 'endclass';
KWD_endclocking: 'endclocking';
KWD_endconfig: 'endconfig';
KWD_endfunction: 'endfunction';
KWD_endgenerate: 'endgenerate';
KWD_endgroup: 'endgroup';
KWD_endinterface: 'endinterface';
KWD_endmodule: 'endmodule';
KWD_endpackage: 'endpackage';
KWD_endprimitive: 'endprimitive';
KWD_endprogram: 'endprogram';
KWD_endproperty: 'endproperty';
KWD_endsequence: 'endsequence';
KWD_endspecify: 'endspecify';
KWD_endtable: 'endtable';
KWD_endtask: 'endtask';
KWD_enum: 'enum';
KWD_event: 'event';
KWD_eventually: 'eventually';
KWD_expect: 'expect';
KWD_export: 'export';
KWD_extends: 'extends';
KWD_extern: 'extern';
KWD_final: 'final';
KWD_first_match: 'first_match';
KWD_for: 'for';
KWD_force: 'force';
KWD_foreach: 'foreach';
KWD_forever: 'forever';
KWD_fork: 'fork';
KWD_forkjoin: 'forkjoin';
KWD_function: 'function';
KWD_generate: 'generate';
KWD_genvar: 'genvar';
KWD_global: 'global';
KWD_highz0: 'highz0';
KWD_highz1: 'highz1';
KWD_if: 'if';
KWD_iff: 'iff';
KWD_ifnone: 'ifnone';
KWD_ignore_bins: 'ignore_bins';
KWD_illegal_bins: 'illegal_bins';
KWD_implements: 'implements';
KWD_implies: 'implies';
KWD_import: 'import';
KWD_incdir: 'incdir';
KWD_include: 'include';
KWD_initial: 'initial';
KWD_inout: 'inout';
KWD_input: 'input';
KWD_inside: 'inside';
KWD_instance: 'instance';
KWD_int: 'int';
KWD_integer: 'integer';
KWD_interconnect: 'interconnect';
KWD_interface: 'interface';
KWD_intersect: 'intersect';
KWD_join: 'join';
KWD_join_any: 'join_any';
KWD_join_none: 'join_none';
KWD_large: 'large';
KWD_let: 'let';
KWD_liblist: 'liblist';
KWD_library: 'library';
KWD_local: 'local';
KWD_localparam: 'localparam';
KWD_logic: 'logic';
KWD_longint: 'longint';
KWD_macromodule: 'macromodule';
KWD_matches: 'matches';
KWD_medium: 'medium';
KWD_modport: 'modport';
KWD_module: 'module';
KWD_nand: 'nand';
KWD_negedge: 'negedge';
KWD_nettype: 'nettype';
KWD_new: 'new';
KWD_nexttime: 'nexttime';
KWD_nmos: 'nmos';
KWD_nor: 'nor';
KWD_noshowcancelled: 'noshowcancelled';
KWD_not: 'not';
KWD_notif0: 'notif0';
KWD_notif1: 'notif1';
KWD_null: 'null';
KWD_or: 'or';
KWD_output: 'output';
KWD_package: 'package';
KWD_packed: 'packed';
KWD_parameter: 'parameter';
KWD_pmos: 'pmos';
KWD_posedge: 'posedge';
KWD_primitive: 'primitive';
KWD_priority: 'priority';
KWD_program: 'program';
KWD_property: 'property';
KWD_protected: 'protected';
KWD_pull0: 'pull0';
KWD_pull1: 'pull1';
KWD_pulldown: 'pulldown';
KWD_pullup: 'pullup';
KWD_pulsestyle_ondetect: 'pulsestyle_ondetect';
KWD_pulsestyle_onevent: 'pulsestyle_onevent';
KWD_pure: 'pure';
KWD_rand: 'rand';
KWD_randc: 'randc';
KWD_randcase: 'randcase';
KWD_randsequence: 'randsequence';
KWD_rcmos: 'rcmos';
KWD_real: 'real';
KWD_realtime: 'realtime';
KWD_ref: 'ref';
KWD_reg: 'reg';
KWD_reject_on: 'reject_on';
KWD_release: 'release';
KWD_repeat: 'repeat';
KWD_restrict: 'restrict';
KWD_return: 'return';
KWD_rnmos: 'rnmos';
KWD_rpmos: 'rpmos';
KWD_rtran: 'rtran';
KWD_rtranif0: 'rtranif0';
KWD_rtranif1: 'rtranif1';
KWD_s_always: 's_always';
KWD_s_eventually: 's_eventually';
KWD_s_nexttime: 's_nexttime';
KWD_s_until: 's_until';
KWD_s_until_with: 's_until_with';
KWD_scalared: 'scalared';
KWD_sequence: 'sequence';
KWD_shortint: 'shortint';
KWD_shortreal: 'shortreal';
KWD_showcancelled: 'showcancelled';
KWD_signed: 'signed';
KWD_small: 'small';
KWD_soft: 'soft';
KWD_solve: 'solve';
KWD_specify: 'specify';
KWD_specparam: 'specparam';
KWD_static: 'static';
KWD_string: 'string';
KWD_strong: 'strong';
KWD_strong0: 'strong0';
KWD_strong1: 'strong1';
KWD_struct: 'struct';
KWD_super: 'super';
KWD_supply0: 'supply0';
KWD_supply1: 'supply1';
KWD_sync_accept_on: 'sync_accept_on';
KWD_sync_reject_on: 'sync_reject_on';
KWD_table: 'table';
KWD_tagged: 'tagged';
KWD_task: 'task';
KWD_this: 'this';
KWD_throughout: 'throughout';
KWD_time: 'time';
KWD_timeprecision: 'timeprecision';
KWD_timeunit: 'timeunit';
KWD_tran: 'tran';
KWD_tranif0: 'tranif0';
KWD_tranif1: 'tranif1';
KWD_tri: 'tri';
KWD_tri0: 'tri0';
KWD_tri1: 'tri1';
KWD_triand: 'triand';
KWD_trior: 'trior';
KWD_trireg: 'trireg';
KWD_type: 'type';
KWD_typedef: 'typedef';
KWD_union: 'union';
KWD_unique: 'unique';
KWD_unique0: 'unique0';
KWD_unsigned: 'unsigned';
KWD_until: 'until';
KWD_until_with: 'until_with';
KWD_untyped: 'untyped';
KWD_use: 'use';
KWD_uwire: 'uwire';
KWD_var: 'var';
KWD_vectored: 'vectored';
KWD_virtual: 'virtual';
KWD_void: 'void';
KWD_wait: 'wait';
KWD_wait_order: 'wait_order';
KWD_wand: 'wand';
KWD_weak: 'weak';
KWD_weak0: 'weak0';
KWD_weak1: 'weak1';
KWD_while: 'while';
KWD_wildcard: 'wildcard';
KWD_wire: 'wire';
KWD_with: 'with';
KWD_within: 'within';
KWD_wor: 'wor';
KWD_xnor: 'xnor';
KWD_xor: 'xor';

//system names (keywords and specials only):
SYSTF_setup: '$setup';
SYSTF_hold: '$hold';
SYSTF_setuphold: '$setuphold';
SYSTF_recovery: '$recovery';
SYSTF_removal: '$removal';
SYSTF_recrem: '$recrem';
SYSTF_skew: '$skew';
SYSTF_timeskew: '$timeskew';
SYSTF_fullskew: '$fullskew';
SYSTF_period: '$period';
SYSTF_width: '$width';
SYSTF_nochange: '$nochange';
SYSTF_error: '$error';
SYSTF_fatal: '$fatal';
SYSTF_info: '$info';
SYSTF_warning: '$warning';
SYSTF_root: '$root';
SYSTF_unit: '$unit';

//identifiers
C_IDENTIFIER: [a-zA-Z_] [a-zA-Z0-9_]*;
IDENTIFIER_EX: [a-zA-Z_] [a-zA-Z0-9_$]+;
ESCAPED_IDENTIFIER: '\\' ~[\n\r\t ]+;
SYSTF_NAME: '$' [a-zA-Z0-9_$]+;

//numeric literals
STRING: '"' (~[\n\r"\\] | '\\' ~[\n\r])* '"';
fragment DECIMAL: [0-9] [0-9_]*;
BASED_DECIMAL: '\'' [Ss]? [Dd] [\t ]* ([XxZz] '_'* | DECIMAL);
BASED_BINARY: '\'' [Ss]? [Bb] [\t ]* [01XxZz?] [01XxZz?_]*;
BASED_OCTAL: '\'' [Ss]? [Oo] [\t ]* [0-7XxZz?] [0-7XxZz?_]*;
BASED_HEX: '\'' [Ss]? [Hh] [\t ]* [0-9A-Fa-fXxZz?] [0-9A-Fa-fXxZz?_]*;
TIME_LITERAL: '1step' | DECIMAL ('.' DECIMAL)? ('s'|'ms'|'us'|'ns'|'ps'|'fs');
REAL_NUMBER: DECIMAL ('.' DECIMAL | ('.' DECIMAL)? [Ee] [+-]? DECIMAL);
UNSIGNED_NUMBER: DECIMAL;
UNBASED_NUMBER: '\'' [01XxZz];

//whitespace
WHITE_SPACE: [\t ]+ -> skip;
ENDLINE: [\r\n]+ -> skip;

//comments
SL_COMMENT: '//' ~[\r\n]* -> skip;
ML_COMMENT: '/*' .*? '*/' -> skip;

//operators
ATAT: '@@';
AT: '@';
WEIGHT: ':=';
SPLIT_WEIGHT: ':/';
SCOPE: '::';
COLON: ':';
COMMA: ',';
DOT: '.';
EQ_CASE: '===';
WILDCARD_EQ: '==?';
EQUAL: '==';
PPATH: '=>';
ASSIGN: '=';
MINUS_ASSIGN: '-=';
NBLK_TRIGGER: '->>';
TRIGGER: '->';
PS_DOWN: '-:';
DEC: '--';
MINUS: '-';
CONS_REP_LBRACK: '[*';
NCONS_REP_LBRACK: '[=';
GOTO_REP_LBRACK: '[->';
LBRACK: '[';
RBRACK: ']';
LCURLY: '{';
RCURLY: '}';
LATTR: '(*';
LPAREN: '(';
RPAREN: ')';
NOL_FOLLOWED_BY: '#-#';
OL_FOLLOWED_BY: '#=#';
CYCLE_DELAY: '##';
DELAY: '#';
QUESTION: '?';
SEMI: ';';
PLUS_ASSIGN: '+=';
PS_UP: '+:';
INC: '++';
PLUS: '+';
NOT_EQ_CASE: '!==';
WILDCARD_NOT_EQ: '!=?';
NOT_EQ: '!=';
LNOT: '!';
RNAND: '~&';
RNOR: '~|';
RXNOR: '~^'|'^~';
BNOT: '~';
SAND: '&&&';
LAND: '&&';
BAND_ASSIGN: '&=';
BAND: '&';
OL_IMPL: '|->';
NOL_IMPL: '|=>';
LOR: '||';
BOR_ASSIGN: '|=';
BOR: '|';
BXOR_ASSIGN: '^=';
BXOR: '^';
POWER: '**';
FPATH: '*>';
RATTR: '*)';
MULT_ASSIGN: '*=';
MULT: '*';
DIV_ASSIGN: '/=';
DIV: '/';
MOD_ASSIGN: '%=';
MOD: '%';
ASL_ASSIGN: '<<<=';
ASL: '<<<';
LEQ: '<->';
SL_ASSIGN: '<<=';
SL: '<<';
LE: '<=';
LT_: '<';
ASR_ASSIGN: '>>>=';
ASR: '>>>';
SR_ASSIGN: '>>=';
SR: '>>';
GE: '>=';
GT: '>';
DOLLAR: '$';
STRUCT_LCURLY: '\'{';
CAST: '\'';
