parser grammar syslogParser;
options
{
tokenVocab=syslogLexer;
}

//main rule
source_text
:
  KWD_package identifier SEMI
  (
    function_declaration
  |
    data_declaration
  |
    parameter_declaration? SEMI
  )*
  KWD_endpackage
  EOF
;

parameter_declaration
:
  ( KWD_parameter | KWD_localparam )
  data_type_or_implicit
  list_of_param_assignments
    #Parameter_declaration_regular
|
  ( KWD_parameter | KWD_localparam )
  KWD_type
  list_of_type_assignments
    #Parameter_declaration_type
;

data_declaration
:
  KWD_const?
  (
    KWD_var data_type_or_implicit
  |
    data_type
  )
  list_of_variable_decl_assignments
  SEMI
|
  type_declaration
;

type_declaration
:
  KWD_typedef
  data_type
  identifier variable_dimension* SEMI
    #Type_declaration_explicit
|
  KWD_typedef
  identifier bit_select? DOT identifier
  identifier SEMI
    #Type_declaration_interfaceAlias
|
  KWD_typedef
  ( KWD_enum | KWD_struct | KWD_union | KWD_interface? KWD_class )?
  identifier SEMI
    #Type_declaration_forward
;

lifetime
:
  KWD_static | KWD_automatic
;

data_type
:
  integer_vector_type signing? packed_dimension*
|
  integer_atom_type signing?
|
  KWD_string
|
  named_type packed_dimension*
;

data_type_or_implicit
:
  data_type 
|
  implicit_data_type
;

implicit_data_type
:
  signing? packed_dimension*
;

class_scope
:
  named_type SCOPE
;

named_type
:
  scoped_identifier parameter_value_assignment?
;

integer_atom_type
:
  KWD_byte | KWD_shortint | KWD_int | KWD_longint | KWD_integer | KWD_time
;
 
integer_vector_type
:
  KWD_bit | KWD_logic | KWD_reg
;

signing
:
  KWD_signed | KWD_unsigned
;

data_type_or_void
:
  data_type | KWD_void
;

list_of_param_assignments
:
  param_assignment ( COMMA param_assignment )*
; 

list_of_tf_variable_identifiers
:
  identifier variable_dimension* ( ASSIGN expression )?
  ( COMMA identifier variable_dimension* ( ASSIGN expression )? )*
;

list_of_type_assignments
:
  type_assignment ( COMMA type_assignment )*
; 

list_of_variable_decl_assignments
:
  variable_decl_assignment ( COMMA variable_decl_assignment )*
;

param_assignment
:
  identifier unpacked_dimension*
  ( ASSIGN param_expression )? 
;

type_assignment
:
  identifier ( ASSIGN data_type )?
;

variable_decl_assignment
:
  identifier variable_dimension* ( ASSIGN expression )?
|
  identifier unsized_dimension variable_dimension* ( ASSIGN dynamic_array_new )?
|
  identifier ( ASSIGN class_new )?
;

class_new
:
  class_scope? KWD_new ( LPAREN list_of_arguments RPAREN )?
|
  KWD_new expression
;

dynamic_array_new
:
  KWD_new LBRACK expression RBRACK ( LPAREN expression RPAREN )?
;

unpacked_dimension
: 
  LBRACK ( range | expression ) RBRACK
;

packed_dimension
: 
  LBRACK range RBRACK
|
  unsized_dimension
;

associative_dimension
:
  LBRACK ( MULT | data_type ) RBRACK 
;

variable_dimension
: 
  unsized_dimension 
|
  unpacked_dimension 
|
  associative_dimension 
;

unsized_dimension
:
  LBRACK RBRACK 
;

function_data_type_or_implicit
:
  data_type_or_void 
|
  implicit_data_type 
;

function_declaration
:
  KWD_function lifetime?
  function_data_type_or_implicit
  identifier
  (
    SEMI
    ( tf_port_declaration | block_item_declaration )*
  |
    LPAREN tf_port_list RPAREN SEMI
    block_item_declaration*
  )
  statement_or_null*
  KWD_endfunction
  ( COLON identifier )?
;

tf_port_list //can be empty
:
  tf_port_item ( COMMA tf_port_item )*
;

tf_port_item //can be empty
:
  tf_port_direction? KWD_var? data_type_or_implicit
  ( identifier variable_dimension* ( ASSIGN expression )? )?
;

tf_port_direction
:
  KWD_input | KWD_output | KWD_inout | KWD_const? KWD_ref
;

tf_port_declaration
:
  tf_port_direction KWD_var? data_type_or_implicit list_of_tf_variable_identifiers SEMI
;

block_item_declaration
:
  data_declaration
;

parameter_value_assignment
:
  DELAY LPAREN ( param_expression ( COMMA param_expression )* )? RPAREN
;

generate_ //!!!
:
  (
    function_declaration //!!!
  |
    KWD_cover zero_delay_or_final LPAREN expression RPAREN statement_or_null //!!!
  )*
;

list_of_variable_assignments
:
  variable_assignment ( COMMA variable_assignment )*
;

assignment_statement
:
  selected_reference ( ASSIGN | LE ) expression SEMI
|
  reference bit_select? ASSIGN ( dynamic_array_new | class_new ) SEMI
|
  selected_reference assignment_operator expression SEMI
;

operator_assignment
:
  selected_reference ASSIGN expression
|
  selected_reference assignment_operator expression
;

assignment_operator
:
  PLUS_ASSIGN | MINUS_ASSIGN | MULT_ASSIGN | DIV_ASSIGN | MOD_ASSIGN |
  BAND_ASSIGN | BOR_ASSIGN | BXOR_ASSIGN | SL_ASSIGN | SR_ASSIGN | ASL_ASSIGN | ASR_ASSIGN 
;

variable_assignment
:
  selected_reference ASSIGN expression
;

seq_block
:
  KWD_begin ( COLON identifier )?
  block_item_declaration*
  statement_or_null*
  KWD_end ( COLON identifier )?
;

statement_or_null
:
  assignment_statement
|
  inc_or_dec_expression SEMI
|
  subroutine_call_statement
|
  jump_statement
|
  seq_block
|
  clocking_drive SEMI //!!!
|
  SEMI
;

jump_statement
:
  KWD_return expression? SEMI 
;

subroutine_call_statement
:
  subroutine_call SEMI
|
  KWD_void CAST LPAREN subroutine_call RPAREN SEMI
;

zero_delay_or_final
:
  DELAY n=UNSIGNED_NUMBER { $n.text.equals("0") }?<fail={"expected 0 delay or 'final'"}>
|
  KWD_final
;

clocking_drive
:
  selected_reference LE cycle_delay? expression
;

cycle_delay
:
  CYCLE_DELAY
  (
    number
  |
    identifier
  |
    LPAREN expression RPAREN
  )
;

concatenation_or_replication
:
  LCURLY expression
  (
    ( COMMA expression )* //concatenation
  |
    LCURLY expression ( COMMA expression )* RCURLY //multiple_concatenation
  )
  RCURLY
;

subroutine_call
:
  scoped_identifier
  ( LPAREN list_of_arguments RPAREN )?
  reference_expr_postfix?
;

list_of_arguments //can be empty
:
  (
    DOT identifier LPAREN expression? RPAREN
  |
    expression? ( COMMA expression? )*
  )
  ( COMMA DOT identifier LPAREN expression? RPAREN )*
;

method_name
:
  identifier | KWD_unique | KWD_and | KWD_or | KWD_xor
;

inc_or_dec_expression
:
  inc_or_dec_operator selected_reference
|
  selected_reference inc_or_dec_operator
;

param_expression
:
  expression | data_type
;

expression
:
  primary
    #Expression_primary
|
  ( PLUS | MINUS | LNOT | BNOT | BAND | RNAND | BOR | RNOR | BXOR | RXNOR )
  primary
    #Expression_unary
|
  LPAREN expression RPAREN
    #Expression_parens
|
  expression
  ( ( PLUS | MINUS ) expression )+
    #Expression_add
|
  expression
  ( ( SL | SR | ASL | ASR ) expression )+
    #Expression_shift
|
  expression
  ( ( LT_ | LE | GT | GE ) expression )+
    #Expression_relation
|
  expression
  ( ( EQUAL | NOT_EQ | EQ_CASE | NOT_EQ_CASE | WILDCARD_EQ | WILDCARD_NOT_EQ ) expression )+
    #Expression_equality
|
  expression
  ( ( BXOR | RXNOR ) expression )+
    #Expression_bxor
;

range
:
  expression COLON expression
;

primary
:
  number
|
  STRING
|
  KWD_null
|
  (
    scoped_identifier
    ( LPAREN list_of_arguments RPAREN )?
  |
    concatenation_or_replication
  |
    LPAREN expression RPAREN
  )
  reference_expr_postfix?
  bit_select?
  part_select?
;

bit_select
:
  ( LBRACK expression RBRACK )+
;

part_select
:
  LBRACK range RBRACK
;

inc_or_dec_operator
:
  INC | DEC 
;

number
:
  UNSIGNED_NUMBER ( BASED_BINARY | BASED_OCTAL | BASED_HEX | BASED_DECIMAL )
    #Integral_number_sized_based
|
  UNSIGNED_NUMBER
    #Integral_number_unsigned
|
  ( BASED_BINARY | BASED_OCTAL | BASED_HEX | BASED_DECIMAL )
    #Integral_number_based
|
  UNBASED_NUMBER
    #Integral_number_logic
;

identifier
:
  C_IDENTIFIER | IDENTIFIER_EX | ESCAPED_IDENTIFIER
;

scoped_identifier
:
  ( identifier parameter_value_assignment? SCOPE )*
  identifier
;

field_select
:
  bit_select? ( DOT identifier bit_select? )* DOT identifier
;

reference
:
  scoped_identifier
  field_select?
;

selected_reference
:
  reference bit_select? part_select?
;

reference_expr_postfix
:
  (
    bit_select?
    part_select?
    DOT
    i=method_name
    ( LPAREN list_of_arguments RPAREN )?
    ( KWD_with LPAREN expression RPAREN )?
  )+
;
